Common

Struct Common 

pub struct Common {
    pub width: Option<f32>,
    pub height: Option<f32>,
    pub flex: Option<f32>,
    pub padding: f32,
    pub overflow: Overflow,
    pub background: Option<Color>,
    pub border: Option<Border>,
    pub keep_with_next: bool,
}
Properties shared by every container/block element: padding, width, height, overflow, background, border, plus flex (taffy-vocabulary, ADR-004) and keep_with_next (ADR-007 / Grundprinzip 9). Deliberately no margin on elements (ADR/03: only padding + Row/Column gap, margin is a Document-level property).

Fields

width: Option<f32>height: Option<f32>flex: Option<f32>

Growth factor along the parent’s main axis; None means “auto”, i.e. sized from measured content. Only has an effect when the parent’s main-axis size is bounded (see lightweight-pdf-layout Row/Column).

padding: f32overflow: Overflowbackground: Option<Color>border: Option<Border>keep_with_next: bool

See plan/05-overflow-and-robustness.md Grundprinzip 9 / ADR-007: only placed if the immediately following sibling also still fits.